"""Module for default pipeline implementation."""

from typing import Dict

from src.detectors.interface import Detector
from src.input_parsers.interface import InputParser
from src.pipeline.interface import Pipeline
from src.replacers.interface import ReplacerInterface
from src.suppressors.interface import Suppressor


class DefaultPipeline(Pipeline):
    """Pipeline that runs the whole anonymization process on the single input."""

    def __init__(
        self,
        input_parser: InputParser,
        detectors: Dict[str, Detector],
        suppressor: Suppressor,
        replacers: Dict[str, ReplacerInterface],
    ):
        """Initialize pipeline.

        Args:
            input_parser (InputParser): Object that parses input into text and
                annotations.
            detectors (Dict[str, Detector]): List of detectors.
            suppressor (Suppressor): List of suppressors.
            replacers (Dict[str, ReplacerInterface]): List of replacers.

        """
        self._input_parser = input_parser
        self._detectors = detectors
        self._suppressor = suppressor
        self._replacers = replacers

    def run(self, input_path: str) -> str:
        """Run the whole anonymization pipeline.

        Args:
            input_path (str): Path to the input supported by input parser.

        Returns:
            str: Anonymized text.

        """
        with open(input_path, "r") as f:
            content = f.read()
        parsed_input = self._input_parser.parse(content)

        detected_entities = []
        for _detector_name, detector in self._detectors.items():
            detected_entities += detector.detect(parsed_input[0], parsed_input[1])

        annotaitons_cleaned = self._suppressor.suppress(detected_entities)

        replaced_input = parsed_input[0]
        annotations_left = annotaitons_cleaned
        for _replacer_name, replacer in self._replacers.items():
            replaced_input, annotations_left = replacer.replace(
                replaced_input, annotations_left
            )

        return replaced_input
